#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from onf_website.component.hearing.interfaces import IHearingInfo, IHearingTarget, HEARING_INFO_KEY
from pyams_content.shared.common.interfaces import IWfSharedContentRoles
from pyams_form.interfaces.form import IFormContextPermissionChecker
from pyams_security.interfaces import IProtectedObject
from pyams_utils.interfaces import VIEW_PERMISSION
from zope.annotation.interfaces import IAnnotations

# import packages
from persistent import Persistent
from pyams_utils.adapter import adapter_config, ContextAdapter
from pyams_utils.traversing import get_parent
from pyramid.threadlocal import get_current_registry
from zope.container.contained import Contained
from zope.interface import implementer
from zope.lifecycleevent import ObjectCreatedEvent
from zope.location import locate
from zope.schema.fieldproperty import FieldProperty


@implementer(IHearingInfo)
class HearingInfo(Persistent, Contained):
    """Hearing info persistent class"""

    _inherit = FieldProperty(IHearingInfo['inherit'])
    _targets = FieldProperty(IHearingInfo['targets'])
    _private = FieldProperty(IHearingInfo['private'])
    _private_groups = FieldProperty(IHearingInfo['private_groups'])
    _source = FieldProperty(IHearingInfo['source'])
    _diffusion_sites = FieldProperty(IHearingInfo['diffusion_sites'])
    _use_location = FieldProperty(IHearingInfo['use_location'])
    _departments = FieldProperty(IHearingInfo['departments'])
    _departments_forests = FieldProperty(IHearingInfo['departments_forests'])
    _domanials_only = FieldProperty(IHearingInfo['domanials_only'])
    _communals_only = FieldProperty(IHearingInfo['communals_only'])
    _forests = FieldProperty(IHearingInfo['forests'])

    @property
    def parent(self):
        return get_parent(self.__parent__, IHearingTarget, allow_context=False)

    @property
    def can_inherit(self):
        return IHearingTarget.providedBy(self.parent)

    @property
    def inherit(self):
        return self._inherit if self.can_inherit else False

    @inherit.setter
    def inherit(self, value):
        if self.can_inherit:
            self._inherit = value

    @property
    def no_inherit(self):
        return not bool(self.inherit)

    @no_inherit.setter
    def no_inherit(self, value):
        self.inherit = not bool(value)

    @property
    def targets(self):
        return IHearingInfo(self.parent).targets if self.inherit else self._targets

    @targets.setter
    def targets(self, value):
        if not (self.can_inherit and self.inherit):
            self._targets = value

    @property
    def private(self):
        return IHearingInfo(self.parent).private if self.inherit else self._private

    @private.setter
    def private(self, value):
        if not (self.can_inherit and self.inherit):
            self._private = value
            security = IProtectedObject(self.__parent__)
            if self._private:
                security.inherit_parent_security = False
                security.everyone_denied = {VIEW_PERMISSION}
                security.authenticated_denied = {VIEW_PERMISSION}
            else:
                security.inherit_parent_security = True
                security.everyone_denied = None
                security.authenticated_denied = None

    @property
    def private_groups(self):
        return IHearingInfo(self.parent).private_groups if self.inherit else self._private_groups

    @private_groups.setter
    def private_groups(self, value):
        if not (self.can_inherit and self.inherit):
            self._private_groups = value
            parent = self.__parent__
            if IWfSharedContentRoles.providedBy(parent):
                if self.private:
                    parent.guests = value
                else:
                    parent.guests = None

    @property
    def source(self):
        return IHearingInfo(self.parent).source if self.inherit else self._source

    @source.setter
    def source(self, value):
        if not (self.can_inherit and self.inherit):
            self._source = value

    @property
    def diffusion_sites(self):
        return IHearingInfo(self.parent).diffusion_sites if self.inherit else self._diffusion_sites

    @diffusion_sites.setter
    def diffusion_sites(self, value):
        if not (self.can_inherit and self.inherit):
            self._diffusion_sites = value

    @property
    def use_location(self):
        return IHearingInfo(self.parent).use_location if self.inherit else self._use_location

    @use_location.setter
    def use_location(self, value):
        if not (self.can_inherit and self.inherit):
            self._use_location = value

    @property
    def ignore_location(self):
        return not self.use_location

    @ignore_location.setter
    def ignore_location(self, value):
        self.use_location = not value

    @property
    def departments(self):
        return IHearingInfo(self.parent).departments if self.inherit else self._departments

    @departments.setter
    def departments(self, value):
        if not (self.can_inherit and self.inherit):
            self._departments = value

    @property
    def departments_forests(self):
        return IHearingInfo(self.parent).departments_forests if self.inherit else self._departments_forests

    @departments_forests.setter
    def departments_forests(self, value):
        if not (self.can_inherit and self.inherit):
            self._departments_forests = value

    @property
    def domanials_only(self):
        return IHearingInfo(self.parent).domanials_only if self.inherit else self._domanials_only

    @domanials_only.setter
    def domanials_only(self, value):
        if not (self.can_inherit and self.inherit):
            self._domanials_only = value

    @property
    def communals_only(self):
        return IHearingInfo(self.parent).communals_only if self.inherit else self._communals_only

    @communals_only.setter
    def communals_only(self, value):
        if not (self.can_inherit and self.inherit):
            self._communals_only = value

    @property
    def forests(self):
        return IHearingInfo(self.parent).forests if self.inherit else self._forests

    @forests.setter
    def forests(self, value):
        if not (self.can_inherit and self.inherit):
            self._forests = value


@adapter_config(context=IHearingTarget, provides=IHearingInfo)
def HearingInfoFactory(target):
    """Hearing info factory"""
    annotations = IAnnotations(target)
    hearing = annotations.get(HEARING_INFO_KEY)
    if hearing is None:
        hearing = annotations[HEARING_INFO_KEY] = HearingInfo()
        get_current_registry().notify(ObjectCreatedEvent(hearing))
        locate(hearing, target, '++hearing++')
    return hearing


@adapter_config(context=IHearingInfo, provides=IFormContextPermissionChecker)
class HearingInfoPermissionChecker(ContextAdapter):
    """Hearing info checker"""

    @property
    def edit_permission(self):
        content = get_parent(self.context, IHearingTarget)
        return IFormContextPermissionChecker(content).edit_permission
