#
# Copyright (c) 2012 Thierry Florac <tflorac AT onf.fr>
# All Rights Reserved.
#


# import standard packages
from datetime import date, timedelta

# import interfaces
from zope.schema.interfaces import ITitledTokenizedTerm

# import packages
from onf_website.reference.client.model import Client
from onf_website.reference.forest.model import PARENT_SCHEMA, PARENT_SESSION
from onf_website.reference.forest.model.code import get_libelle
from onf_website.reference.forest.model.interfaces.foret import IAbstractBaseForetRegroupementModel, IRegroupementModel, \
    IRegroupement, IForetModel, IForet
from onf_website.reference.insee.model import Departement, Commune
from onf_website.reference.orga.model import Structure
from pyams_alchemy import Base
from pyams_alchemy.engine import get_user_session
from pyams_alchemy.mixin import DynamicSchemaMixin
from pyams_utils.unicode import translate_string
from sqlalchemy import Column, Unicode, Integer, Numeric, Date, Boolean
from sqlalchemy.ext.declarative import AbstractConcreteBase, declared_attr
from sqlalchemy.orm import relation, relationship
from sqlalchemy.orm.session import object_session
from sqlalchemy.schema import ForeignKey, PrimaryKeyConstraint
from sqlalchemy.sql.expression import func
from zope.interface import provider, implementer


# Table REGION_IFN

class RegionIFN(DynamicSchemaMixin, Base):
    """IFN region class"""

    __tablename__ = 'region_ifn'
    __schema__ = PARENT_SCHEMA

    code = Column(Unicode(3), primary_key=True)
    codifn = Column(Unicode(4))
    libelle = Column(Unicode(50))

    @property
    def title(self):
        return '%s - %s' % (self.code, self.libelle)


# Polymorphic base classes

@provider(IAbstractBaseForetRegroupementModel)
@implementer(ITitledTokenizedTerm)
class AbstractBaseForetRegroupement(AbstractConcreteBase, Base):
    """Base model class for forests and groups"""

    __entity__ = 'RGPT+FRT'

    id_nat_frt = Column(Unicode(7), primary_key=True)
    code_frt = Column(Unicode(25))
    date_saisie = Column(Date)
    date_debut_validite = Column(Date)
    date_fin_validite = Column(Date)
    est_domaniale = Column(Boolean)
    regime = Column(Unicode(2))
    cadre_legal_gestion = Column(Unicode(2))
    frais_garderie = Column(Unicode(3))
    libelle_preposition = Column(Unicode(3))
    libelle_strict = Column(Unicode(60))
    libelle_usage = Column(Unicode(120))
    libelle_recherche = Column(Unicode(120))
    surface_cadastrale = Column(Numeric(15, 4))
    code_service_gestionnaire = Column('service_gestionnaire', Unicode(6))
    code_structure_rh_gestionnaire = Column('structure_rh_gestionnaire', Unicode(6))
    code_structure_rh_sign = Column('structure_rh_sign', Unicode(6))
    centre_profit = Column(Unicode(10))
    commentaire = Column(Unicode(250))

    # declared attributes
    @declared_attr
    def departement(self):
        return Column(Unicode(5), ForeignKey(Departement.dep))

    @declared_attr
    def departement_insee(self):
        return relationship(Departement)

    @declared_attr
    def code_region_ifn(self):
        return Column('region_ifn', Unicode(3), ForeignKey(RegionIFN.code))

    @declared_attr
    def region_ifn(self):
        return relationship(RegionIFN)

    @property
    def value(self):
        return self.id_nat_frt

    @property
    def token(self):
        return self.id_nat_frt

    @property
    def title(self):
        return u'%s (%s)' % (self.libelle_usage, self.departement)

    @property
    def longname(self):
        return u'%s (%s)' % (self.libelle_usage, self.departement_insee.nccenr)

    @property
    def invalide(self):
        return self.date_fin_validite < date.today()

    @property
    def date_debut_invalidite(self):
        return self.date_fin_validite + timedelta(days=1)

    @property
    def libelle_cadre_gestion(self):
        return get_libelle(u'CADRE_LEGAL_GESTION', self.cadre_legal_gestion) if self.cadre_legal_gestion else u''

    @property
    def libelle_classement(self):
        return get_libelle(u'CLASSEMENT_AGRICOLE', self.frais_garderie) if self.frais_garderie else u''

    @property
    def preposition(self):
        return get_libelle(u'LIBELLE_PREPOSITION', self.libelle_preposition) if self.libelle_preposition else u'--'

    @property
    def service_gestionnaire(self):
        return Structure.get(self.code_service_gestionnaire).first()

    @property
    def libelle_service_gestionnaire(self):
        return self.service_gestionnaire.title if self.code_service_gestionnaire else u'--'

    @property
    def structure_rh_gestionnaire(self):
        return Structure.get(self.code_structure_rh_gestionnaire).first()

    @property
    def libelle_structure_gestionnaire(self):
        return self.structure_rh_gestionnaire.title if self.code_structure_rh_gestionnaire else u'--'

    @property
    def inactive(self):
        return (self.cadre_legal_gestion == 'NG') or (self.date_fin_validite < date.today())

    @classmethod
    def find(cls, query, session=PARENT_SESSION):
        return Regroupement.find(query, session), Foret.find(query, session)

    @classmethod
    def find_by_id(cls, id_nat_frt, session=PARENT_SESSION):
        return Regroupement.find_by_id(id_nat_frt, session), Foret.find_by_id(id_nat_frt, session)

    @classmethod
    def find_by_label(cls, query, session=PARENT_SESSION):
        return Regroupement.find_by_label(query, session), Foret.find_by_label(query, session)


# Table RDF_REGROUPEMENT

@provider(IRegroupementModel)
@implementer(IRegroupement)
class Regroupement(DynamicSchemaMixin, AbstractBaseForetRegroupement):
    """Model class for RDF_REGROUPEMENT table"""

    __tablename__ = 'rdf_regroupement'
    __schema__ = PARENT_SCHEMA
    __mapper_args__ = {'polymorphic_identity': 'regroupement',
                       'concrete': True}

    __entity__ = 'RGPT'

    id_nat_frt = Column(Unicode(7), primary_key=True)

    @classmethod
    def get(cls, code, session=PARENT_SESSION):
        if isinstance(code, str):
            code = code.split(',')
        session = get_user_session(session)
        return session.query(Regroupement).filter(Regroupement.id_nat_frt.in_(code))

    @classmethod
    def find(cls, query, session=PARENT_SESSION):
        if isinstance(query, dict):
            id_nat_frt = query.get('id_nat_frt')
            label = query.get('label')
        else:
            query = query.strip()
            if len(query) == 7:
                try:
                    _check = int(query[1:6])
                except:
                    id_nat_frt = None
                    label = query
                else:
                    id_nat_frt = query
                    label = None
            else:
                id_nat_frt = None
                label = query
        session = get_user_session(session)
        if id_nat_frt:
            return Regroupement.find_by_id(id_nat_frt, session)
        elif label:
            return Regroupement.find_by_label(label, session)

    @classmethod
    def find_by_id(cls, id_nat_frt, session=PARENT_SESSION):
        if isinstance(id_nat_frt, (list, tuple)):
            params = Regroupement.id_nat_frt.in_(id_nat_frt)
        else:
            params = Regroupement.id_nat_frt == id_nat_frt
        session = get_user_session(session)
        return session.query(Regroupement).filter(params)

    @classmethod
    def find_by_label(cls, query, session=PARENT_SESSION):
        query = '%' + translate_string(query, keep_chars="_-.'").upper() + '%'
        if '-' in query:
            query = query.replace('-', ' ')
        session = get_user_session(session)
        return session.query(Regroupement) \
                      .filter(Regroupement.libelle_recherche.like(query))

    @property
    def communes(self):
        session = object_session(self)
        return session.query(ForetSurCommune.code_insee,
                             ForetSurCommune.libelle_insee,
                             func.sum(ForetSurCommune.surface_cadastrale)) \
                      .join(Foret, ForetSurCommune.id_nat_frt == Foret.id_nat_frt) \
                      .join(Regroupement, Foret.id_nat_frt_regroupement == Regroupement.id_nat_frt) \
                      .filter(Regroupement.id_nat_frt == self.id_nat_frt) \
                      .group_by(ForetSurCommune.code_insee,
                                ForetSurCommune.libelle_insee) \
                      .order_by(ForetSurCommune.libelle_insee)


# Table RDF_FORET

@provider(IForetModel)
@implementer(IForet)
class Foret(DynamicSchemaMixin, AbstractBaseForetRegroupement):
    """Model class for RDF_FORET table"""

    __tablename__ = 'rdf_foret'
    __schema__ = PARENT_SCHEMA
    __mapper_args__ = {'polymorphic_identity': 'foret',
                       'concrete': True}

    __entity__ = 'FRT'

    id_nat_frt = Column(Unicode(7), primary_key=True)
    date_expiration_contrat = Column(Date)
    surface_retenue_gestion = Column(Numeric(15, 4))
    surface_sylviculture = Column(Numeric(15, 4))
    code_tgpe = Column(Unicode(13))
    surface_tgpe = Column(Numeric(15, 4))
    pefc_num_cert = Column(Unicode(15))
    pefc_debut_cert = Column(Date)
    pefc_fin_cert = Column(Date)
    marquage_pefc = Column(Boolean)
    pefc_num_marq = Column(Unicode(15))
    pefc_debut_marq = Column(Date)
    pefc_fin_marq = Column(Date)
    date_fin_gestion = Column(Date)
    id_nat_frt_regroupement = Column(Unicode(7), ForeignKey(Regroupement.id_nat_frt))
    date_liaison_groupement = Column(Date)

    regroupement = relation(Regroupement, backref='forets')

    @classmethod
    def get(cls, code, session=PARENT_SESSION):
        if isinstance(code, str):
            code = code.split(',')
        session = get_user_session(session)
        return session.query(Foret).filter(Foret.id_nat_frt.in_(code))

    @classmethod
    def find(cls, query, session=PARENT_SESSION):
        if isinstance(query, dict):
            id_nat_frt = query.get('id_nat_frt')
            label = query.get('label')
        else:
            query = query.strip()
            if len(query) == 7:
                try:
                    _check = int(query[1:6])
                except:
                    id_nat_frt = None
                    label = query
                else:
                    id_nat_frt = query
                    label = None
            else:
                id_nat_frt = None
                label = query
        session = get_user_session(session)
        if id_nat_frt:
            return Foret.find_by_id(id_nat_frt, session)
        elif label:
            return Foret.find_by_label(label, session)

    @classmethod
    def find_by_id(cls, id_nat_frt, session=PARENT_SESSION):
        if isinstance(id_nat_frt, (list, tuple)):
            params = Foret.id_nat_frt.in_(id_nat_frt)
        else:
            params = Foret.id_nat_frt == id_nat_frt
        session = get_user_session(session)
        return session.query(Foret).filter(params)

    @classmethod
    def find_by_label(cls, query, session=PARENT_SESSION):
        query = '%' + translate_string(query, keep_chars="_-.'").upper() + '%'
        if '-' in query:
            query = query.replace('-', ' ')
        session = get_user_session(session)
        return session.query(Foret) \
                      .filter(Foret.libelle_recherche.like(query))

    @property
    def libelle_chorus(self):
        return self.code_tgpe or u'--'


# Table RDF_COMMUNE

class ForetSurCommune(DynamicSchemaMixin, Base):
    """Model class for RDF_COMMUNE table"""

    __tablename__ = 'rdf_commune'
    __schema__ = PARENT_SCHEMA

    @declared_attr
    def __table_args__(cls):
        return (
            PrimaryKeyConstraint('code_insee', 'id_nat_frt'),
            cls.get_schema()
        )

    code_insee = Column(Unicode(5), ForeignKey(Commune.code))
    id_nat_frt = Column(Unicode(7), ForeignKey(Foret.id_nat_frt))
    libelle_insee = Column(Unicode(70))
    surface_cadastrale = Column(Numeric(15, 4))

    commune = relation(Commune, backref='forets')
    foret = relation(Foret, backref='communes')


# Table RDF_PROPRIETAIRE

class ProprietaireForet(DynamicSchemaMixin, Base):
    """Model class for RDF_PROPRIETAIRE table"""

    __tablename__ = 'rdf_proprietaire'
    __schema__ = PARENT_SCHEMA

    @declared_attr
    def __table_args__(cls):
        return (
            PrimaryKeyConstraint('clecli', 'id_nat_frt'),
            cls.get_schema()
        )

    clecli = Column(Unicode(30), ForeignKey(Client.clecli))
    id_nat_frt = Column(Unicode(7), ForeignKey(Foret.id_nat_frt))
    categorie = Column(Unicode(2))
    pourcentage_part = Column(Numeric(5, 2))
    type_copropriete = Column(Unicode(3))
    type_lot = Column(Unicode(2))
    commentaire = Column('commentaire_rdf', Unicode(250))
    numerateur_part = Column(Integer)
    denominateur_part = Column(Integer)

    client = relation(Client)
    foret = relation(Foret, backref='proprietaires')

    @property
    def libelle_type(self):
        return get_libelle(u'TYPE_COPROPRIETE', self.type_copropriete) if self.type_copropriete else u''

    @property
    def libelle_categorie(self):
        return get_libelle(u'CATEGORIE_PROPRIETAIRE', self.categorie) if self.categorie else u''


# Table RDF_REPRESENTANT

class RepresentantForet(DynamicSchemaMixin, Base):
    """Model class for RDF_REPRESENTANT table"""

    __tablename__ = 'rdf_representant'
    __schema__ = PARENT_SCHEMA

    @declared_attr
    def __table_args__(cls):
        return (
            PrimaryKeyConstraint('clecli', 'id_nat_frt'),
            cls.get_schema()
        )

    clecli = Column(Unicode(30), ForeignKey(Client.clecli))
    id_nat_frt = Column(Unicode(7), ForeignKey(Foret.id_nat_frt))
    categorie = Column(Unicode(2))
    commentaire = Column('commentaire_rdf', Unicode(250))

    client = relation(Client)
    foret = relation(Foret, backref='representants')

    @property
    def libelle_categorie(self):
        return get_libelle(u'REPRESENTANT_PROPRIETAIRE', self.categorie) if self.categorie else u''

RepresentantForet.regroupement = relation(Regroupement,
                                          foreign_keys=RepresentantForet.id_nat_frt,
                                          primaryjoin=RepresentantForet.id_nat_frt == Regroupement.id_nat_frt,
                                          backref='representants')
